@@ -35,9 +35,16 @@ summary.satscan = function(object, ...) {
   stopifnot(class(object)=="satscan")
     
   cat(object$main[c(9:11, 15:20,22)], fill=1)
+  
+  if(is.data.frame(object$col)) {
     
-  cat(paste0("There were ", dim(object$col)[1], " clusters identified."), fill=1)
-  cat(paste0("There were ", sum(object$col$P_VALUE < .05), " clusters with p < .05."))
+    cat(paste0("There were ", dim(object$col)[1], " clusters identified."), fill=1)
+    cat(paste0("There were ", sum(object$col$P_VALUE < .05), " clusters with p < .05."))
+  } else {
+    
+    cat("No cluster dBase file detected.", fill=1)
+    cat("Run SaTScan with 'MostLikelyClusterEachCentroidDBase' parameter set to 'y' for cluster information.")
+  }
   
   invisible(object)
 }
