@@ -29,7 +29,8 @@ subin = function (x,ssparams) {
     if (length(which(substr(ssparams,1,regexpr("=",ssparams)) == inprm)) == 0) {
       param.name = substr(inprm,1,regexpr("=",inprm)-1)
       
-      warning("Trouble! There is no parameter '", param.name, "'.", call.=FALSE, immediate.=TRUE)
+      # warning("Trouble! There is no parameter '", param.name, "'.", call.=FALSE, immediate.=TRUE)
+      warning(paste0("Parameter ", param.name, " either does not exist or needs to be specified in ss.options.extra()"), call.=FALSE, immediate.=TRUE)
       
       earliest.parameter.set = search_for_param(ssenv, inprm)
       if(earliest.parameter.set != "") {
@@ -133,18 +134,40 @@ charlistopts = function (x) {
 
 #Huge ups to http://digitheadslabnotebook.blogspot.com/2011/06/environments-in-r.html
 #which helped me get the scoping to play out correctly.
-#ss.options will: 1) return the current values of .ss.params, if no invals
-#                 2) Reset the values of .ss.params, if reset==TRUE
-#                 3) change the values of the listed parameters, if a) invals = 
-#                      c("param=value","param=value") or list(param="value")
+#
 #' @title Set or reset parameters to be used by SaTScan
 #' 
-#' @description Set or reset parameters to be used by SaTScan
+#' @description Set or reset parameters used by SaTScan using the built-in
+#'   rsatscan parameter list. Some parameters must be specified using the
+#'   ss.options.extra() function.
+#'
+#'   Refer to the SaTScan parameter file after generating it from the SaTScan
+#'   GUI. The parameter file can be viewed in a text editor.
 #' 
 #' @details \code{ss.options()} is intended to function like \code{par()} or 
 #' \code{options()}.  There is a default set of parameter settings that resembles 
 #' the one used by SaTScan, except that it produces all possible output files and
 #' makes them as .dbf files instead of text.
+#'
+#' @section SaTScan Versions:
+#' The \code{version} argument defines which parameter set the script uses, 
+#' not necessarily the version of SaTScan being used to execute the analyses. 
+#' SaTScan is backwards compatible with older versions of parameter sets. 
+#' For instance you might create a script that uses the 10.1 parameter set. 
+#' That parameter set in the script will continue to work as you upgrade your SaTScan 
+#' executable to newer versions. This is the same way that rsatscan worked up to version 1.0.3 
+#' where the script was locked to the 9.2 parameter set but you still could use SaTScan 9.3, 
+#' 9.4, 9.7, 10.1, etc without access to the newer parameter set options introduced in
+#' those versions. As such, users with scripts created with rsatscan prior to version 1.0.4 
+#' must explicitly set the parameter set version in their scripts.
+#' 
+#' @section Environment Objects:
+#' The parameter sets are stored in the 'ssenv' environment object.
+#' 
+#' WARNING: Clearing your R environment will delete the 'ssenv' object and cause an error
+#' when attempting to use any SaTScan parameter sets. The 'rsatscan' library must
+#' be reloaded to restore the 'ssenv' object and allow SaTScan parameters to work
+#' correctly.
 #' 
 #' @param invals A list with entries of the form name=value, where value should be 
 #' in quotes unless it is a number. Alternatively, may be a character vector whose
@@ -157,30 +180,11 @@ charlistopts = function (x) {
 #' parameter set. If this parameter is NULL or not specified, then parameters 
 #' are reset based on the latest version of SaTScan.
 #' 
-#' This parameter defines which parameter set the script uses, 
-#' not necessarily the version of SaTScan being used to execute the analyses. 
-#' SaTScan is backwards compatible with older versions of parameter sets. 
-#' For instance you might create a script that uses the 10.1 parameter set. 
-#' That parameter set in the script will continue to work as you upgrade your SaTScan 
-#' executable to newer versions. This is the same way that rsatscan worked up to version 1.0.3 
-#' where the script was locked to the 9.2 parameter set but you still could use SaTScan 9.3, 
-#' 9.4, 9.7, 10.1, etc without access to the newer parameter set options introduced in
-#' those versions. As such, users with scripts created with rsatscan prior to version 1.0.4 
-#' must explicitly set the parameter set version in their scripts.
-#' 
-#' The parameter sets are stored in the 'ssenv' environment object.
-#' 
-#' WARNING: Clearing your R environment will delete the 'ssenv' object and cause an error
-#' when attempting to use any SaTScan parameter sets. The 'rsatscan' library must
-#' be reloaded to restore the 'ssenv' object and allow SaTScan parameters to work
-#' correctly.
-#' 
-#' @return If \code{invals == NULL}, returns the current parameter set, 
-#' as altered by previous 
-#' calls to \code{ss.options()} since the last call with \code{reset=TRUE}.  Otherwise 
-#' returns modified parameter set invisibly.  The side effect, if \code{invals != NULL}, is to 
-#' set the current values of the parameters per the value of \code{invals} 
-#' and \code{reset}.
+#' @returns If `invals == NULL`, returns the current parameter set, as altered
+#'   by previous calls to `ss.options()` since the last call with `reset=TRUE`.
+#'   Otherwise returns modified parameter set invisibly. The side effect, if
+#'   `invals != NULL`, is to set the current values of the parameters per the
+#'   value of `invals` and `reset`.
 #' 
 #' @export
 #' 
@@ -234,6 +238,7 @@ ss.options = function (invals=NULL, reset=FALSE, version=NULL) {
     invisible(ssenv$.ss.params)
   }
 }
+
 # review the help text for logic-- matches function??
 
 #I need to think about how this will work when called by another function.
@@ -243,33 +248,67 @@ ss.options = function (invals=NULL, reset=FALSE, version=NULL) {
 
 #' @title Add lines to the current SaTScan parameter list
 #' 
-#' @description Allows you to add arbitrary lines to the current set
-#' of SaTScan parameters
-#' 
-#' @details For certain SaTScan models or inputs (multiple data sets, 
-#' Polygon),
-#' SaTScan allows a variable number of parameters; these 
-#' parameters are not used/allowed for other models or inputs.
-#' This function allows the user to add 
-#' arbitrary lines to the current list of 
-#' parameters.  In addition to the options mentioned, it could also be
-#' used to add comments to the parameter file.
+#' @description This function allows the user to add lines to the current list
+#'   of parameters. It can be used to add parameters that are not allowed using
+#'   the ss.options() function or to add comments to the parameter file.
+#'   
+#' @details The SaTScan parameters and corresponding sections that must be added
+#'   to the current parameter list using ss.options.extra() are listed below.
+#'   
+#'   |                                           |                                            |
+#'   |-------------------------------------------|--------------------------------------------|
+#'   | **`Section=”Input”`**                     |                                            |
+#'   |                                           |                                            |
+#'   | `Casefile-SourceLinelistFieldMap=`        | Comma separated list of variables in the   |
+#'   |                                           | case input data to be included in the line |
+#'   |                                           | list output file. For each variable,       |
+#'   |                                           | includes the column number, variable       |
+#'   |                                           | type, and variable name, separated by      |
+#'   |                                           | colons.                                    |
+#'   |                                           |                                            |
+#'   |                                           |                                            |
+#'   | **`Section=”Multiple Datasets”`**         |                                            |
+#'   |                                           |                                            |
+#'   | `[FileType][X]=`                          | Analogous to parameters in the Input       |
+#'   | `[FileType][X]-SourceFieldMap=`           | section, repeated for every additional     |
+#'   | `[FileType][X]-SourceDelimiter=`          | dataset, where \[FileType] is CaseFile,    |
+#'   | `[FileType][X]-SourceFirstRowHeader=`     | ControlFile, or PopulationFile, and \[X]   |
+#'   | `[FileType][X]-SourceLinelistFieldMap=`   | may be between 2 and 20.                   |
+#'   |                                           |                                            |
+#'   |                                           |                                            |
+#'   | **`Section=”Polygons”`**                  |                                            |
+#'   |                                           |                                            |
+#'   | `Polygon[X]=`                             | The bound region for each Polygon,         |
+#'   |                                           | when using the Continuous Poisson          |
+#'   |                                           | probability model.                         |
 #' 
 #' @param invals A character vector, which will be added to the current parameter list.
-#' @param section A character vector of length 1 that specifies the section of the
-#' parameter file to add the new parameters to. Sections are denoted in the 'ssenv'
-#' object by square brackets. 
-#' 
-#' @return Nothing.  
+#' @param section A character vector of length 1 that specifies the section of
+#'   the parameter file to add the new parameters to. Sections are denoted in
+#'   the 'ssenv' object by square brackets.
+#' @param new.section A logical variable indicating that a new section in the
+#'   parameter file should be created. (default = FALSE)
+#'
+#' @returns `ss.options.extra()` returns `NULL` and adds lines to the parameter
+#'   set per the values of invals and section.
 #' 
 #' @export
-#' 
+#'
 #' @examples
 #' \dontrun{
 #' # Append second data file to the Multiple Data Sets section of the parameter list
 #' ss.options.extra(invals=list(CaseFile2="NYCfever.cas"), section="Multiple Data Sets")
 #' print(ss.options()[67:70])
 #' 
+#' # Specify columns in the case input file to be included in the line list output file 
+#' ss.options.extra(invals=list('CaseFile-SourceLinelistFieldMap'=strwrap(
+#'                                 "0:0:\"IndividualID\",
+#'                                  4:1:\"DescriptiveLongitude\",
+#'                                  2:2:\"DescriptiveLatitude\",
+#'                                  3:3:\"AGE\",
+#'                                  1:3:\"GENDER\"")),
+#'                  section="Input")
+#' 
 #' # Can also append to the end of the parameter file by not specifying a section.
 #' # This is useful for adding comments.
 #' # Note that the input value can be specified as a character string instead of a list 
@@ -277,8 +316,7 @@ ss.options = function (invals=NULL, reset=FALSE, version=NULL) {
 #' ss.options.extra(invals=";This is the end of the parameter list.")
 #' tail(ss.options(), 3)
 #' }
-#' 
-ss.options.extra = function(invals=NULL, section=NULL) {
+ss.options.extra = function(invals=NULL, section=NULL, new.section=FALSE) {
   
   if (is.null(invals)) stop("This function doesn't do anything when there is no input")
   if(!inherits(invals, "list") && !inherits(invals, "character")) stop("Please input a character vector or list for 'invals'")
@@ -288,38 +326,53 @@ ss.options.extra = function(invals=NULL, section=NULL) {
     invals = charlistopts(invals)
   }
   
-  if(is.null(section)) {
-      
-    #if no section of the parameter list is specified, simply append to the end of the parameter list
-    #this is useful for adding comments/notes
-    ssenv$.ss.params =  c(ssenv$.ss.params, invals)
-  } else {
-      
+  if(!is.null(section)) {
+    
     if(!inherits(section, "character")) {
-        
+      
       stop("'section' must be a character string.")
     }
-      
+    
     if(length(section) > 1) {
-        
+      
       stop("Only one section can be specified.")
     }
-      
+    
     #add brackets to section name if not present already
     if(!(startsWith(section, "[") && endsWith(section, "]"))) {
-        
+      
       section = paste0("[", section, "]")
     }
+    
+    #check if section is in parameter list
+    section.exists <- (tolower(section) %in% tolower(ssenv$.ss.params))
+    
+    if(new.section) {
       
-    #error if section is not in parameter list
-    if(!(tolower(section) %in% tolower(ssenv$.ss.params))) {
+      if(!section.exists) {
         
-      stop(paste0("Section '", section, "' not found in parameter list."))
-    }
+        #Add new section to the parameter file
+        ssenv$.ss.params <- c(ssenv$.ss.params, section, "")
+      } else {
+        
+        message(paste0("Section ", section, " already exists in the parameter list. Parameter(s) have been added to the existing section."))
+      }
+    } else {
       
+      if(!section.exists) {
+        
+        stop(paste0("Section '", section, "' not found in the parameter list. To add a new section to the parameter list, set new.section to TRUE."))
+      }
+    }
+    
     #add parameter(s) to selected section
     insertion.index <- match(tolower(section), tolower(ssenv$.ss.params))
     ssenv$.ss.params = append(ssenv$.ss.params, invals, after=insertion.index)
+  } else {
+    
+    #if no section of the parameter list is specified, simply append to the end of the parameter list
+    #this is useful for adding comments/notes
+    ssenv$.ss.params =  c(ssenv$.ss.params, invals)
   }
     
   invisible()
@@ -327,10 +380,6 @@ ss.options.extra = function(invals=NULL, section=NULL) {
 # for help page: examples of [Polygon] and Multiple Data Sets
 
 
-
-
-
-
 # Functions to write out the param file
 # Probably a really bad idea to make matchout = FALSE-- only useful to write file
 # from R but examine output manually
