@@ -97,10 +97,10 @@ read.sci = function(location,file) foreign::read.dbf(paste0(location,file,".sci.
 #' probably nor necessary to even have it as a function.
 #' 
 #' @param location A directory location, including the trailing "/"
-#' @param file A file name, without the extension.
+#' @param file A file name, with any extensions.
 #' 
 #' @return A data frame.
 read.satscanmain = function(location,file) suppressWarnings(
-  readLines(paste0(location,file,".txt")))
+  readLines(paste0(location,file)))
 
 # need to get these to read dates correctly.  Currently reading as factors
\ No newline at end of file
