@@ -14,9 +14,6 @@
 #' useful if you expect SaTScan to take a long time to run.
 #' 
 #' @details The parameter file may have been made by the \code{\link{ss.options}} function or not.  
-#' If not, or if the \code{matchout = FALSE} parameter was set in \code{ss.options}, then the
-#' return object will include the main text output from SaTScan only you manually set the 
-#' \code{ResultsFile} SaTScan parameter to have the same name as the parameter file.
 #' 
 #' @return A satscan-class object, which is a list of 8 items, not all of which are always made, depending on SaTScan options and whether the program call was successful or not:
 #' \describe{
@@ -50,7 +47,7 @@
 satscan = function(
   prmlocation,  prmfilename,  sslocation = "c:/progra~2/satscan",
   ssbatchfilename = "SaTScanBatch", cleanup = TRUE, verbose=FALSE) {
-    
+
     ssfile = paste0(stripslash(sslocation), "/", ssbatchfilename)
     if (Sys.which(ssfile) =="")
           stop("SaTScan is not there or is not runnable")
@@ -76,37 +73,56 @@ satscan = function(
     #   warning.message <- paste0("Warning: You are running SaTScan version ", satscan.version.string, " with parameters defined for version ", spec.version.string)
     #   warning(warning.message)
     # }
+ 
+    status <- system(paste(shQuote(ssfile), shQuote(infile)), show.output.on.console=verbose)
     
-    system(paste(shQuote(ssfile), shQuote(infile)), show.output.on.console=verbose)
+    if(status != 0 && !verbose) {
+      
+      warning("Error running SaTScan executable. Please use verbose = TRUE for more information.")
+    }
     
-    mainfile = if  (file.exists(paste0(prmloc,prmfilename,".txt")))
-      read.satscanmain(prmloc,prmfilename) else NA
-    colfile = if  (file.exists(paste0(prmloc,prmfilename,".col.dbf")))
-      read.col(prmloc,prmfilename) else NA
-    rrfile = if  (file.exists(paste0(prmloc,prmfilename,".rr.dbf")))
-      read.rr(prmloc,prmfilename) else NA
-    gisfile = if  (file.exists(paste0(prmloc,prmfilename,".gis.dbf")))
-      read.gis(prmloc,prmfilename) else NA
-    llrfile = if  (file.exists(paste0(prmloc,prmfilename,".llr.dbf")))
-      read.llr(prmloc,prmfilename) else NA
-    scifile = if  (file.exists(paste0(prmloc,prmfilename,".sci.dbf")))
-      read.sci(prmloc,prmfilename) else NA  
+    resultsfile.line <- prm[startsWith(prm, "ResultsFile")]
+    resultsfilepath <- strsplit(resultsfile.line, "=", fixed=TRUE)[[1]][2]
+    
+    resultsfile <- basename(resultsfilepath)
+    if(resultsfile == resultsfilepath) {
+      
+      resultsfileloc <- prmloc
+    } else {
+      
+      resultsfileloc <- paste0(dirname(resultsfilepath), "/")
+    }
+
+    resultsfilename <- sub("\\.[^\\.]*$", "", resultsfile)
+    
+    mainfile = if  (file.exists(paste0(resultsfileloc,resultsfile)))
+      read.satscanmain(resultsfileloc,resultsfile) else NA
+    colfile = if  (file.exists(paste0(resultsfileloc,resultsfilename,".col.dbf")))
+      read.col(resultsfileloc,resultsfilename) else NA
+    rrfile = if  (file.exists(paste0(resultsfileloc,resultsfilename,".rr.dbf")))
+      read.rr(resultsfileloc,resultsfilename) else NA
+    gisfile = if  (file.exists(paste0(resultsfileloc,resultsfilename,".gis.dbf")))
+      read.gis(resultsfileloc,resultsfilename) else NA
+    llrfile = if  (file.exists(paste0(resultsfileloc,resultsfilename,".llr.dbf")))
+      read.llr(resultsfileloc,resultsfilename) else NA
+    scifile = if  (file.exists(paste0(resultsfileloc,resultsfilename,".sci.dbf")))
+      read.sci(resultsfileloc,resultsfilename) else NA  
     if (verbose) cat("\n \n Any following message is from st_read() in the sf package \n")
-      shpfile = if (file.exists(paste0(prmloc,prmfilename,".col.shp"))) {
+      shpfile = if (file.exists(paste0(resultsfileloc,resultsfilename,".col.shp"))) {
         if (requireNamespace("sf", quietly = TRUE)) {
-          sf::st_read(dsn=stripslash(prmloc), layer=paste0(prmfilename,".col"),
+          sf::st_read(dsn=stripslash(resultsfileloc), layer=paste0(resultsfilename,".col"),
                     quiet=!verbose) 
                 }  else cat("\n \n sf package not installed, so shapefile can't be imported")
           } else NA
     if (cleanup) {
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".txt")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".col.dbf")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".rr.dbf")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".gis.dbf")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".llr.dbf")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".sci.dbf")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".col.shp")))
-        suppressWarnings(file.remove(paste0(prmloc,prmfilename,".col.shx")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfile)))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".col.dbf")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".rr.dbf")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".gis.dbf")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".llr.dbf")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".sci.dbf")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".col.shp")))
+        suppressWarnings(file.remove(paste0(resultsfileloc,resultsfilename,".col.shx")))
       }
   return(structure(list(main=mainfile, col=colfile, rr = rrfile, gis=gisfile, llr=llrfile, 
                         sci=scifile, shapeclust=shpfile, prm=prm), class="satscan"))
